@@ -23,22 +23,25 @@ import pytest
 
 sys.path.append(os.path.dirname(os.path.dirname(__file__)))
 
-from tests.utils import tester, subscript_item, in_finally, start_position, end_position
+from .utils import tester, subscript_item, in_finally, start_position, end_position
 
 PYPY = 'pypy' in sys.version.lower()
-PY3 = sys.version_info[0] == 3
 
 from executing import Source, only, NotOneValueFound
-from executing.executing import get_instructions, function_node_types
+from executing.executing import NodeFinder, get_instructions, function_node_types
 
 from executing._exceptions import VerifierFailure, KnownIssue
 
-if sys.version_info >= (3,11):
-    from tests.deadcode import Deadcode
+from tests.deadcode import is_deadcode
 
 if eval("0"):
     global_never_defined = 1
 
+if sys.version_info[:2] == (3, 9):
+    # https://github.com/alexmojaki/executing/pull/71#issuecomment-1723634310
+    import asttokens.util
+    asttokens.util.fstring_positions_work = lambda: True
+
 
 def calling_expression():
     frame = inspect.currentframe().f_back.f_back
@@ -319,10 +322,8 @@ class TestStuff(unittest.TestCase):
         check(u'# coding=utf8\né', 'gbk', exception=UnicodeDecodeError)
         check(u'# coding=gbk\né', 'utf8', matches=False)
 
-        # In Python 3 the default encoding is assumed to be UTF8
-        if PY3:
-            check(u'é', 'utf8')
-            check(u'é', 'gbk', exception=SyntaxError)
+        check(u'é', 'utf8')
+        check(u'é', 'gbk', exception=SyntaxError)
 
     def test_multiline_strings(self):
         tester('a')
@@ -376,7 +377,7 @@ class TestStuff(unittest.TestCase):
     def assert_qualname(self, func, qn, check_actual_qualname=True):
         qualname = Source.for_filename(__file__).code_qualname(func.__code__)
         self.assertEqual(qn, qualname)
-        if PY3 and check_actual_qualname:
+        if check_actual_qualname:
             self.assertEqual(qn, func.__qualname__)
         self.assertTrue(qn.endswith(func.__name__))
 
@@ -571,10 +572,38 @@ class TestStuff(unittest.TestCase):
                 a(b(c()))
 
     def test_listcomp(self):
-        if sys.version_info >= (3, 11):
+        if (3, 11) <= sys.version_info < (3, 12):
+            # ListComp is inlined in 3.12
             result = [calling_expression() for e in [1]]
             self.assertIsInstance(result[0], ast.ListComp)
 
+    def test_iter(self):
+        class iter_test:
+            def __init__(self, typ):
+                self.typ = typ
+                self.it = iter([1, 2])
+
+            def __iter__(self):
+                assert isinstance(calling_expression(), self.typ)
+                return self
+
+            def __next__(self):
+                assert isinstance(calling_expression(), self.typ)
+                return next(self.it)
+
+        assert list(iter_test(ast.Call)) == [1, 2]
+        assert next(iter(iter_test(ast.Call))) == 1
+
+        if sys.version_info >= (3, 11):
+
+            assert [i for i in iter_test(ast.ListComp)] == [1, 2]
+            assert {i for i in iter_test(ast.SetComp)} == {1, 2}
+            assert {i: i for i in iter_test(ast.DictComp)} == {1: 1, 2: 2}
+            assert list(i for i in iter_test(ast.GeneratorExp)) == [1, 2]
+
+            for i in iter_test(ast.For):
+                assert i in (1, 2)
+
     def test_decorator_cache_instruction(self):
         frame = inspect.currentframe()
 
@@ -597,7 +626,10 @@ class TimeOut(Exception):
     pass
 
 
-def dump_source(source, start, end, context=4, file=sys.stdout):
+def dump_source(source, start, end, context=4, file=None):
+    if file is None:
+        file= sys.stdout
+
     print("source code:", file=file)
     start = max(start.lineno - 5, 0)
     num = start + 1
@@ -648,9 +680,9 @@ def is_annotation(node):
 
 
 
-def sample_files():
+def sample_files(samples):
     root_dir = os.path.dirname(__file__)
-    samples_dir = os.path.join(root_dir, "samples")
+    samples_dir = os.path.join(root_dir, samples)
 
     for filename in os.listdir(samples_dir):
         full_filename = os.path.join(samples_dir, filename)
@@ -669,6 +701,48 @@ def sample_files():
         yield pytest.param(full_filename, result_filename, id=filename)
 
 
+@pytest.mark.parametrize(
+    "full_filename,result_filename", list(sample_files("small_samples"))
+)
+@pytest.mark.skipif(sys.version_info<(3,),reason="no 2.7 support")
+def test_small_samples(full_filename, result_filename):
+    skip_sentinel = [
+        "load_deref",
+        "4851dc1b626a95e97dbe0c53f96099d165b755dd1bd552c6ca771f7bca6d30f5",
+        "508ccd0dcac13ecee6f0cea939b73ba5319c780ddbb6c496be96fe5614871d4a",
+        "fc6eb521024986baa84af2634f638e40af090be4aa70ab3c22f3d022e8068228",
+        "42a37b8a823eb2e510b967332661afd679c82c60b7177b992a47c16d81117c8a",
+        "206e0609ff0589a0a32422ee902f09156af91746e27157c32c9595d12072f92a",
+    ]
+
+    skip_annotations = [
+        "d98e27d8963331b58e4e6b84c7580dafde4d9e2980ad4277ce55e6b186113c1d",
+        "9b3db37076d3c7c76bdfd9badcc70d8047584433e1eea89f45014453d58bbc43",
+    ]
+
+    if any(s in full_filename for s in skip_sentinel) and sys.version_info < (3, 11):
+        pytest.xfail("SentinelNodeFinder does not find some of the nodes (maybe a bug)")
+
+    if any(s in full_filename for s in skip_annotations) and sys.version_info < (3, 7):
+        pytest.xfail("no `from __future__ import annotations`")
+
+    if (
+        (sys.version_info[:2] == (3, 7))
+        and "ad8aa993e6ee4eb5ee764d55f2e3fd636a99b2ecb8c5aff2b35fbb78a074ea30"
+        in full_filename
+    ):
+        pytest.xfail("(i async for i in arange) can not be analyzed in 3.7")
+
+    if (
+        (sys.version_info[:2] == (3, 5) or PYPY)
+        and "1656dc52edd2385921104de7bb255ca369713f4b8c034ebeba5cf946058109bc"
+        in full_filename
+    ):
+        pytest.skip("recursion takes to long in 3.5")
+
+    TestFiles().check_filename(full_filename, check_names=True)
+
+
 
 
 @pytest.mark.skipif(
@@ -677,7 +751,7 @@ def sample_files():
 )
 class TestFiles:
 
-    @pytest.mark.parametrize("full_filename,result_filename", list(sample_files()))
+    @pytest.mark.parametrize("full_filename,result_filename", list(sample_files("samples")))
     def test_sample_files(self, full_filename, result_filename):
 
         self.start_time = time.time()
@@ -703,6 +777,12 @@ class TestFiles:
                 filename = inspect.getsourcefile(module)
             except TypeError:
                 continue
+
+            except AttributeError as error:
+                if str(error)== "'_SixMetaPathImporter' object has no attribute '_path'":
+                    # work around for https://github.com/benjaminp/six/issues/376
+                    continue
+                raise
     
             if not filename:
                 continue
@@ -738,6 +818,10 @@ class TestFiles:
 
 
     def check_filename(self, filename, check_names):
+
+        # increase the recursion limit in testing mode, because there are files out there with large ast-nodes
+        # example: tests/small_samples/1656dc52edd2385921104de7bb255ca369713f4b8c034ebeba5cf946058109bc.py
+        sys.setrecursionlimit(3000)
         source = Source.for_filename(filename)
 
         if source.tree is None:
@@ -745,17 +829,8 @@ class TestFiles:
             print("skip %s"%filename)
             return
 
-        if sys.version_info >= (3,11):
-            Deadcode.annotate(source.tree)
-
         print("check %s"%filename)
 
-        if PY3 and sys.version_info < (3, 11):
-            code = compile(source.text, filename, "exec", dont_inherit=True)
-            for subcode, qualname in find_qualnames(code):
-                if not qualname.endswith(">"):
-                    code_qualname = source.code_qualname(subcode)
-                    assert code_qualname == qualname
 
         nodes = defaultdict(list)
         decorators = defaultdict(list)
@@ -776,12 +851,22 @@ class TestFiles:
                 decorators[(node.lineno, node.name)] = []
 
         try:
-            code = compile(source.tree, source.filename, "exec")
+            code = compile(source.tree, source.filename, "exec", dont_inherit=True)
         except SyntaxError:
             # for example:
             # SyntaxError: 'return' outside function
             print("skip %s" % filename)
             return
+        except RecursionError:
+            print("skip %s" % filename)
+            return 
+
+        if sys.version_info < (3, 11):
+            for subcode, qualname in find_qualnames(code):
+                if not qualname.endswith(">"):
+                    code_qualname = source.code_qualname(subcode)
+                    assert code_qualname == qualname
+
         result = list(self.check_code(code, nodes, decorators, check_names=check_names))
 
         if not re.search(r'^\s*if 0(:| and )', source.text, re.MULTILINE):
@@ -821,6 +906,12 @@ class TestFiles:
 
                     if is_literal(node):
                         continue
+
+                    if len(values) == 0 and is_deadcode(node):
+                        continue
+
+                    if isinstance(node,ast.Name) and node.id=="__debug__":
+                        continue
                 else:
                     # x (is/is not) None
                     none_comparison = (
@@ -899,7 +990,7 @@ class TestFiles:
                         # "%50s"%(a,) is missing an BUILD_STRING instruction which normally maps to BinOp
                         continue
 
-                    if getattr(node, "deadcode", False):
+                    if len(values)==0 and is_deadcode(node):
                         continue
 
                     if (
@@ -909,6 +1000,21 @@ class TestFiles:
                     ):
                         continue
 
+                if sys.version_info >= (3, 12):
+                    if (
+                        isinstance(node, ast.Call)
+                        and isinstance(node.func, ast.Name)
+                        and node.func.id == "super"
+                    ):
+                        # super optimization
+                        continue
+
+                    if isinstance(node, ast.Name) and isinstance(
+                        node.parent, ast.TypeAlias
+                    ):
+                        # type alias names have no associated bytecode
+                        continue
+
                 if sys.version_info >= (3, 10):
                     correct = len(values) >= 1
                 elif sys.version_info >= (3, 9) and in_finally(node):
@@ -927,6 +1033,7 @@ class TestFiles:
                     p("correct:", correct)
                     p("len(values):", len(values))
                     p("values:", values)
+                    p("deadcode:", is_deadcode(node))
 
                     p()
 
@@ -960,31 +1067,33 @@ class TestFiles:
                     )
                     p()
 
-                    p("all bytecodes in this range:")
+                    if sys.version_info >= (3, 11):
+                        p("all bytecodes in this range:")
 
-                    bc = compile(source.text, filename, "exec")
 
-                    def inspect(bc):
-                        first = True
-                        for i in dis.get_instructions(bc):
+                        bc = compile(source.text, filename, "exec")
 
-                            if (
-                                i.positions.lineno is not None
-                                and i.positions.lineno <= node.end_lineno
-                                and node.lineno <= i.positions.end_lineno
-                            ):
-                                if first:
-                                    p("block name:", bc.co_name)
-                                    first = False
-                                p(i.positions, i.opname, i.argval)
+                        def inspect(bc):
+                            first = True
+                            for i in dis.get_instructions(bc):
 
-                        for const in bc.co_consts:
-                            if isinstance(const, types.CodeType):
-                                inspect(const)
+                                if (
+                                    i.positions.lineno is not None
+                                    and i.positions.lineno <= node.end_lineno
+                                    and node.lineno <= i.positions.end_lineno
+                                ):
+                                    if first:
+                                        p("block name:", bc.co_name)
+                                        first = False
+                                    p(i.positions, i.opname, i.argval)
 
-                    inspect(bc)
+                            for const in bc.co_consts:
+                                if isinstance(const, types.CodeType):
+                                    inspect(const)
 
-                    self.fail()
+                        inspect(bc)
+
+                    raise AssertionError()
 
         return result
 
@@ -1032,10 +1141,14 @@ class TestFiles:
                     and inst.opname
                     in (
                         "LOAD_GLOBAL",
+                        "LOAD_FROM_DICT_OR_DEREF",
+                        "LOAD_SUPER_ATTR",
                         "STORE_ATTR",
                         "DELETE_ATTR",
                         "DELETE_NAME",
+                        "DELETE_DEREF",
                         "DELETE_FAST",
+                        "DELETE_GLOBAL",
                         "STORE_SUBSCR",
                         "STORE_SLICE",
                         "DELETE_SUBSCR",
@@ -1070,6 +1183,11 @@ class TestFiles:
                 if inst.positions.lineno == None:
                     continue
 
+            if sys.version_info >= (3,12):
+                if inst.opname == "CALL_INTRINSIC_1" and inst.argrepr=='INTRINSIC_LIST_TO_TUPLE':
+                    # convert list to tuple
+                    continue
+
             frame = C()
             frame.f_lasti = inst.offset
             frame.f_code = code
@@ -1120,7 +1238,7 @@ class TestFiles:
                             )
                         )
 
-                self.fail()
+                raise
 
             except Exception as e:
                 # continue for every case where this can be an known issue
@@ -1179,7 +1297,9 @@ class TestFiles:
                 # Attributes which appear ambiguously in modules:
                 #   op1.sign, op2.sign = (0, 0)
                 #   nm_tpl.__annotations__ = nm_tpl.__new__.__annotations__ = types
-                if not py11 and inst.opname == 'STORE_ATTR' and inst.argval in ['sign', '__annotations__']:
+                if not py11 and inst.opname == 'STORE_ATTR' and inst.argval in [
+                    'sign', '__annotations__', '__deprecated__'
+                ]:
                     continue
 
                 if inst.opname == 'LOAD_FAST' and inst.argval == '.0':
@@ -1194,6 +1314,14 @@ class TestFiles:
                 ):
                     continue
 
+                if (
+                    sys.version_info >= (3, 12)
+                    and inst.positions.col_offset == inst.positions.end_col_offset == 0
+                    and inst.argval
+                    in ("__type_params__", ".type_params", "__classdict__")
+                ):
+                    continue
+
                 # report more information for debugging
                 print("mapping failed")
 
@@ -1254,32 +1382,10 @@ class TestFiles:
 
                 raise
 
-            if py11 and getattr(node, "deadcode", True) == True:
-                print("code is alive:")
-                print("There is a bug in the deadcode detection, because the python compiler has generated bytecode for this node.")
-
-                print("deadcode:",getattr(node, "deadcode", "<undefined>"))
-
-                print("node:",ast.dump(node))
-                print("line:",node.lineno)
-
-                non_dead_parent=node
-                while getattr(non_dead_parent,"deadcode",True)==True:
-                    non_dead_parent=non_dead_parent.parent
-
-                dump_source(source.text, start_position(non_dead_parent), end_position(non_dead_parent))
-
-                from deadcode import dump_deadcode
-
-                
-                dump_deadcode(non_dead_parent)
-
-                self.fail()
-
             # `argval` isn't set for all relevant instructions in python 2
             # The relation between `ast.Name` and `argval` is already
             # covered by the verifier and much more complex in python 3.11 
-            if isinstance(node, ast.Name) and (PY3 or inst.argval) and not py11:
+            if isinstance(node, ast.Name) and not py11:
                 assert inst.argval == node.id, (inst, ast.dump(node))
 
             if ex.decorator:
